@@ -73,15 +73,16 @@ as.seglist.default<-function(x, ...) stop("Not yet implemented!")
 #' @seealso \code{\link{ngraph},\link{igraph}}
 #' @export
 #' @method as.seglist igraph
+#' @importFrom igraph is.directed is.connected graph.dfs degree
 #' @rdname seglist
 as.seglist.igraph<-function(x, origin=NULL, Verbose=FALSE, ...){
   # Handle degenerate cases
   if(!is.connected(x)) stop("Graph is not fully connected!")
-  if(igraph::vcount(x)==0) {
+  if(vcount(x)==0) {
     if(Verbose) warning("Empty graph! Seglist not defined")
     return(NULL)
   }
-  if(igraph::is.directed(x) && !igraph::is.dag(x)){
+  if(is.directed(x) && !igraph::is.dag(x)){
     stop("Graph has cycles!")
   }
   
@@ -92,11 +93,11 @@ as.seglist.igraph<-function(x, origin=NULL, Verbose=FALSE, ...){
   # having separate program logic
   vids=igraph::V(x)$vid
   if(is.null(vids)){
-    vids=seq.int(igraph::vcount(x))
+    vids=seq.int(vcount(x))
   }
   
   # Floating point
-  if(igraph::vcount(x)==1) {
+  if(vcount(x)==1) {
     return(seglist(vids))
   }
   
@@ -120,20 +121,23 @@ as.seglist.igraph<-function(x, origin=NULL, Verbose=FALSE, ...){
   }
   
   # Now do a depth first search to ensure that ordering is correct
-  dfs=igraph::graph.dfs(x, root=origin, father=TRUE, neimode='all')
-  ncount=igraph::degree(x)
+  dfs=graph.dfs(x, root=origin, father=TRUE, neimode='all')
+  # cache orders for speed: dfs$order[i] is slooooow in igraph>=1.0
+  orders=as.integer(dfs$order)
+  ncount=degree(x)
   # put the first vertex into the first segment
   # note that here and elsewhere the points stored in curseg will be the
   # _original_ vertex ids specified by "vid" attribute of input graph
-  curseg=vids[dfs$order[1]]
+  curseg=vids[orders[1]]
   if(length(ncount)==1) stop("Unexpected singleton point found!")
   sl=seglist()
   # we have more than 1 point in graph and some work to do!
+  fathers=as.integer(dfs$father)
   for(i in seq.int(from=2,to=length(dfs$order))){
-    curpoint=dfs$order[i]
+    curpoint=orders[i]
     if(length(curseg)==0){
       # segment start, so initialise with parent
-      curseg=vids[dfs$father[curpoint]]
+      curseg=vids[fathers[curpoint]]
     }
     # always add current point
     curseg=c(curseg,vids[curpoint])
