@@ -21,6 +21,9 @@ test_that("c.neuronlist behaves", {
   c610.nodf=Cell07PNs[6:10]
   attr(c610.nodf,'df')=NULL
   expect_equivalent(c(Cell07PNs[1:5], c610.nodf), Cell07PNs[1:10])
+  
+  expect_equivalent(kcs20[1:6], c(kcs20[1:2], kcs20[3:4], kcs20[5:6]), 
+                    "combine more than 2 neuronlists")
   expect_error(c(Cell07PNs[1:5], NULL))
   expect_error(c(Cell07PNs[1:5], Cell07PNs[1:5]), "neurons with the same name")
 })
@@ -191,9 +194,22 @@ test_that("basic interactive 3d functionality",{
     "NIA8L", "NIA8R", "NNA9L", "NNC4R", "NNE1L", "OFD2L", "SDD8L", 
     "TKC8R")
   expect_equal(find.neuron(selfun, db=Cell07PNs), sel_neuron)
+  # NB equivalent because in one case the attributes on the attached data.frame 
+  # are kept, in the other case not. This a pretty obscure difference and not
+  # one that I can sort out in a hurry.
+  expect_equivalent(find.neuron(selfun, db=Cell07PNs, rval='data.frame'), 
+               Cell07PNs[sel_neuron,])
+  expect_equal(find.neuron(selfun, db=Cell07PNs, rval='neuronlist'), 
+                    Cell07PNs[sel_neuron])
   
   sel_soma=c("EBH20L", "EBH20R", "EBJ3R", "EBO15L", "EBO53L")
   expect_equal(find.soma(selfun, db=Cell07PNs), sel_soma)
+  expect_equal(find.soma(selfun, db=Cell07PNs, invert = TRUE),
+               setdiff(names(Cell07PNs), sel_soma))
+  expect_equivalent(find.soma(selfun, db=Cell07PNs, rval='data.frame'), 
+                    Cell07PNs[sel_soma,])
+  expect_equal(find.soma(selfun, db=Cell07PNs, rval='neuronlist'), 
+               Cell07PNs[sel_soma])
   
   rgl.close()
 })
