@@ -1,7 +1,7 @@
 context("npop3d")
 
 test_that("pop last plotted neurons",{
-  open3d()
+  nopen3d()
   original.rgl.ids=rgl.ids()
   plot3d(c("EBH11R", "EBH20L"), db=Cell07PNs)
   npop3d()
