@@ -99,6 +99,17 @@ if(!is.null(cmtk.bindir())){
                     ncol=3, byrow = T)
     expect_equal(tm, baseline)
   })
+  
+  test_that("we can mirror some image data", {
+    reg="testdata/cmtk/FCWB_mirror_level-01.list"
+    img="testdata/nrrd/FCWB_2um_mask.nrrd"
+    bim=boundingbox(read.im3d(img, ReadData = F))
+    tf=tempfile(fileext = '.nrrd')
+    on.exit(unlink(tf))
+    mirror(img, output=tf, target=img)
+    expect_true(file.exists(tf))
+    expect_equal(boundingbox(tf), bim)
+  })
 }
 
 test_that("we can mirror using different forms of axis specification", {
